@@ -1,4 +1,4 @@
-from collections import defaultdict, Counter
+from collections import defaultdict, Counter, deque
 from pathlib import Path
 from typing import Type
 
@@ -35,7 +35,7 @@ class RFXmlFormatter(FormatterBase):
         try:
             data = document.sourceText
             encoding = document.properties.get('encoding', "UTF-8") if document.properties else "UTF-8"
-            if document.sourceText and document.boundaries and document.annotations:
+            if document.sourceText and document.boundaries:
                 if data.lower().startswith("<?xml"):
                     decl, data = data.split("?>", maxsplit=1)
                     decl += "?>"
@@ -43,7 +43,7 @@ class RFXmlFormatter(FormatterBase):
                 baseNs = root.nsmap.get(None, None)
                 # Ignore all namespaces
                 for el in root.iter():
-                    if baseNs and el.tag.startswith(f"{{{baseNs}}}"):
+                    if baseNs and el.tag not in [ET.Comment, ET.PI, ET.Entity] and el.tag.startswith(f"{{{baseNs}}}"):
                         _, _, el.tag = el.tag.rpartition('}')
                 doc_annexes = root.find("DOC_ANNEXES")
                 if not doc_annexes:
@@ -62,26 +62,38 @@ class RFXmlFormatter(FormatterBase):
                     r = root.xpath(boundary.name)
                     if len(r) == 1:
                         node = r[0]
-                        boundaries[node] = Range.closedOpen(boundary.start, boundary.end)
+                        boundaries[node] = 0
                         buckets[Range.closedOpen(boundary.start, boundary.end)] = node
-                for a in document.annotations:
-                    annotation = Annotation(**a) if isinstance(a, dict) else a
-                    form = document.text[annotation.start:annotation.end]
-                    if buckets.contains(annotation.start) and buckets.contains(annotation.end):
-                        zones = buckets[Range.closedOpen(annotation.start, annotation.end)]
-                        for zone in zones:
-                            for term in annotation.terms:
-                                ident = term.identifier if parameters.absolute_uri\
-                                    else term.identifier[term.identifier.index("#") + 1:]
-                                terms[zone][ident].append(form)
 
-                def left_longest_match(item):
-                    r: Range = item[1]
-                    return r.lowerEndpoint(), r.upperEndpoint() - r.lowerEndpoint()
+                # compute depth first order of boundaries
+                queue = deque([root])
+                index = 0
+                while queue:
+                    el = queue.popleft()
+                    if el in boundaries:
+                        boundaries[el] = index
+                    queue.extend(el)
+                    index = index + 1
 
-                sorted_boundaries = sorted(boundaries.items(), key=left_longest_match, reverse=False)
+                if document.annotations:
+                    for a in document.annotations:
+                        annotation = Annotation(**a) if isinstance(a, dict) else a
+                        form = document.text[annotation.start:annotation.end]
+                        if buckets.contains(annotation.start) and buckets.contains(annotation.end):
+                            zones = buckets[Range.closedOpen(annotation.start, annotation.end)]
+                            for zone in zones:
+                                for term in annotation.terms:
+                                    ident = term.identifier if parameters.absolute_uri\
+                                        else term.identifier[term.identifier.index("#") + 1:]
+                                    terms[zone][ident].append(form)
 
-                for node, r in sorted_boundaries:
+                def depth_first_order(item):
+                    node_index = item[1]
+                    return node_index
+
+                sorted_boundaries = sorted(boundaries.items(), key=depth_first_order, reverse=False)
+
+                for node, ignored in sorted_boundaries:
                     if node in terms and terms[node]:
                         descripteurs = ET.Element("DESCRIPTEURS_" + node.tag, node.attrib)
                         for ident, forms in terms[node].items():
@@ -98,12 +110,14 @@ class RFXmlFormatter(FormatterBase):
                             descripteurs.append(descripteur)
                         thesaurus.append(descripteurs)
 
-                data = ET.tostring(root, pretty_print=True, encoding=encoding)
+                data = ET.tostring(root, pretty_print=True, encoding=encoding, xml_declaration=True)
             filename = "file.xml"
             if document.properties and "fileName" in document.properties:
                 filepath = Path(document.properties['fileName'])
                 filename = f"{filepath.stem}.xml"
-            resp = Response(content=data, media_type="application/xml")
+            media_type = "application/xml" if encoding.lower() == "utf-8" \
+                else f"application/xml; charset={encoding.lower()}"
+            resp = Response(content=data, media_type=media_type)
             resp.headers["Content-Disposition"] = f"attachment; filename={filename}"
             resp.charset = encoding
             return resp
