@@ -16,6 +16,11 @@ def test_files():
     return testdir.glob('*.json')
 
 
+def descriptors_mix_test_files():
+    testdir = Path(__file__).parent / 'data'
+    return testdir.glob('descriptors_mix*.json')
+
+
 def test_no_sourceText():
     with pytest.raises(HTTPException) as err:
         formatter = RFXmlFormatter()
@@ -33,16 +38,16 @@ def test_xml_rf(test_file):
         options = RFXmlParameters()
         resp: Response = formatter.format(doc, options)
         assert resp.status_code == 200
-        assert resp.media_type == "application/xml"
+        assert resp.media_type.startswith("application/xml")
+        result = test_file.with_suffix(".xml")
+        with result.open("wb") as fout:
+            fout.write(resp.body)
         root = ET.fromstring(resp.body)
         baseNs = root.nsmap.get(None, None)
         all_descs = list(root.iterdescendants(f"{{{baseNs}}}DESCRIPTEUR"))
         all_forms = list(root.iterdescendants(f"{{{baseNs}}}FORME"))
         assert len(all_descs) > 0
         assert len(all_forms) > 0
-        result = test_file.with_suffix(".xml")
-        with result.open("wb") as fout:
-            fout.write(resp.body)
 
 
 @pytest.mark.parametrize('test_file', test_files())
@@ -54,7 +59,7 @@ def test_xml_rf_without_forms(test_file):
         options = RFXmlParameters(with_forms=False, absolute_uri=False)
         resp: Response = formatter.format(doc, options)
         assert resp.status_code == 200
-        assert resp.media_type == "application/xml"
+        assert resp.media_type.startswith("application/xml")
         content_type, content_parameters = parse_options_header(resp.headers['Content-Disposition'])
         if b'filename' in content_parameters:
             filename = content_parameters[b'filename'].decode("utf-8")
@@ -67,3 +72,29 @@ def test_xml_rf_without_forms(test_file):
             assert "#" not in desc.attrib['Id']
         assert len(all_descs) > 0
         assert len(all_forms) == 0
+
+
+@pytest.mark.parametrize('test_file', descriptors_mix_test_files())
+def test_xml_rf_descriptors_mix(test_file):
+    with test_file.open("r") as fin:
+        docs = json.load(fin)
+        doc = Document(**docs[0])
+        formatter = RFXmlFormatter()
+        options = RFXmlParameters(with_forms=True, absolute_uri=False)
+        resp: Response = formatter.format(doc, options)
+        assert resp.status_code == 200
+        assert resp.media_type.startswith("application/xml")
+        content_type, content_parameters = parse_options_header(resp.headers['Content-Disposition'])
+        if b'filename' in content_parameters:
+            filename = content_parameters[b'filename'].decode("utf-8")
+        assert Path(doc.properties['fileName']).stem == Path(filename).stem
+        root = ET.fromstring(resp.body)
+        ns = {'rf': 'http://www.grouperf.com'}
+        subs = root.findall("./rf:DOC_ANNEXES/rf:ANNOTATIONS_THESAURUS/*", ns)
+        unique_subs = []
+        for sub in subs:
+            tag = sub.tag[len('{http://www.grouperf.com}DESCRIPTEURS_'):]
+            if len(unique_subs) == 0 or unique_subs[-1] != tag:
+                unique_subs.append(tag)
+
+        assert unique_subs == ['DOC_INTRO', 'DOC_DEV', 'SECTION'] or unique_subs == ['DOC_DEV', 'SECTION']
